@@ -1,4 +1,5 @@
-
+from typing import NamedTuple, Dict, Any, Union, Optional, List, Tuple, Callable, Type, Sequence, Iterable, Iterator
+from pathlib import Path
 from collections import OrderedDict, namedtuple
 
 from .typing import unspecified_argument
@@ -16,6 +17,16 @@ class Registry(OrderedDict):
 		self[name] = obj
 		return obj
 
+	class NotFoundError(KeyError): pass
+
+
+	def find(self, x, default=unspecified_argument):
+		if x in self:
+			return self[x]
+		if default is not unspecified_argument:
+			return default
+		raise self.NotFoundError(x)
+
 
 	def is_registered(self, obj):
 		for opt in self.values():
@@ -45,9 +56,12 @@ class Entry_Registry(Registry):
 	'''
 	Automatically wraps data into an "entry" object (namedtuple) which is stored in the registry
 	'''
+	entry_cls: namedtuple = None
 	def __init_subclass__(cls, key_name='name', components=[], required=[]):
 		super().__init_subclass__()
-		cls.entry_cls = namedtuple(f'{cls.__name__}_Entry', [key_name, *components])
+		entry_keys = [key_name, *components]
+		# cls._entry_keys = entry_keys
+		cls.entry_cls: namedtuple = namedtuple(f'{cls.__name__}_Entry', entry_keys)
 		cls._required_keys = [key_name, *required]
 		cls._key_name = key_name
 
@@ -103,7 +117,7 @@ class Double_Registry(Registry):
 			return self.backward()[x]
 		if default is not unspecified_argument:
 			return default
-		raise KeyError(x)
+		raise self.NotFoundError(x)
 
 
 	def backward(self):
@@ -131,7 +145,7 @@ class Double_Registry(Registry):
 
 class Entry_Double_Registry(Double_Registry, Entry_Registry):
 	
-	def __init_subclass__(cls, primary_component='name', sister_component='cls', components=[], required=[]):
+	def __init_subclass__(cls, primary_component='name', sister_component='value', components=[], required=[]):
 		super().__init_subclass__(key_name=primary_component, components=[sister_component, *components],
 		                          required=[sister_component, *required])
 		cls._sister_key_name = sister_component
@@ -150,6 +164,10 @@ class Entry_Double_Registry(Double_Registry, Entry_Registry):
 		return getattr(value, cls._key_name)
 
 
+	def get_value(self, key):
+		return getattr(self.find(key), self._sister_key_name)
+
+
 	def update(self, other, sync=True):
 		if sync:
 			self._sister_registry_object.update({self._get_sister_entry_key(k, v): v
@@ -169,24 +187,22 @@ class Entry_Double_Registry(Double_Registry, Entry_Registry):
 		return super().__delitem__(key, sync=False)
 
 
-	@classmethod
-	def _get_decorator_class(cls):
-		return cls.DecoratorBase
-
-
-	def get_decorator(self, name=None):
+	def get_decorator(self, name: Optional[str] = None, defaults: Dict[str, Any] = None):
+		if defaults is None:
+			defaults = {}
 		return type(f'{self.__class__.__name__}_Decorator' if name is None else name,
-		            (self._get_decorator_class(),), {'_registry': self})
+		            (self.DecoratorBase,), {'_registry': self, '_defaults': defaults})
 
 
 	class DecoratorBase:
 		_registry = None
+		_defaults = None
 		def __init__(self, *args, **kwargs):
 
-			registry = self._get_registry()
+			registry = self._registry
 
 			arg_keys = list(registry.entry_cls._fields)
-			del arg_keys[1]
+			# del arg_keys[1]
 			args = dict(zip(arg_keys, args))
 
 			overlap = ', '.join({registry._sister_key_name, *args.keys()}.intersection(kwargs))
@@ -196,16 +212,18 @@ class Entry_Double_Registry(Double_Registry, Entry_Registry):
 			self.params = {**args, **kwargs}
 
 
-		@classmethod
-		def _get_registry(cls):
-			return cls._registry
+		# @classmethod
+		# def _get_registry(cls):
+		# 	return cls._registry
 
 
 		def _register(self, val, **params):
-			registry = self._get_registry()
+			registry = self._registry
 			key = registry._sister_key_name
 			if key not in params:
 				params[key] = val
+			full = self._defaults.copy()
+			full.update(params)
 			return registry.new(**params)
 
 
@@ -214,13 +232,37 @@ class Entry_Double_Registry(Double_Registry, Entry_Registry):
 			return sister_value
 
 
+class Path_Registry(Entry_Double_Registry, sister_component='path'):
+	def __init_subclass__(cls, sister_component='path', components=[], required=[]):
+		super().__init_subclass__(primary_component='name', sister_component=sister_component,
+		                          components=components, required=required)
 
-class Class_Registry(Entry_Double_Registry):
+	def new(self, name, path, *args, **kwargs):
+		if isinstance(path, str):
+			path = Path(path)
+		return super().new(name, path, *args, **kwargs)
+
+	def get_path(self, key):
+		return self.get_value(key)
+
+
+class Function_Registry(Entry_Double_Registry, sister_component='fn'):
+	def __init_subclass__(cls, sister_component='fn', components=[], required=[]):
+		super().__init_subclass__(primary_component='name', sister_component=sister_component,
+		                          components=components, required=required)
+
+	def get_function(self, key):
+		return self.get_value(key)
+
+
+class Class_Registry(Entry_Double_Registry, sister_component='cls'):
 
 	def __init_subclass__(cls, sister_component='cls', components=[], required=[]):
 		super().__init_subclass__(primary_component='name', sister_component=sister_component,
 		                          components=components, required=required)
 
+	def get_class(self, key):
+		return self.get_value(key)
 
 	class DecoratorBase(Entry_Double_Registry.DecoratorBase):
 		def _register(self, val, name=None, **params):
